function test_slunecni_soustava()
%     Slunce    Merkur       Venuse        Zeme          Mars          Jupiter       Saturn         Uran           Neptun
m  = [1.9885e30 3.30200e23   4.8685e24     5.97237e24    6.4185e23     1.899e27      5.6846e26      8.6832e25      1.02430e26    ]; % [kg]
x0 = [0         4.6001272e10 1.07476002e11 1.47098074e11 2.06644545e11 7.40742598e11 13.49467375e11 27.35555035e11 44.52940833e11]; % [m]
v0 = [0         5.898e4      3.5259e4      3.0287e4      2.6499e4      13.705e3      11.82e3        7.128e3        5.479e3       ]; % [m s^-1]
co_= {'r'       'k'          'g'           'b'           'r'           'm'           'c'            'm'            'b'           };             
G  = 6.6743015e-11;    % [m^3 kg^-1 s^-2]
[m,p] = sort(m);   x0 = x0(p);   v0 = v0(p);   nx = size(x0,2);
co = co_;
for i =1:nx
  co{i} = co_{p(i)};
endfor
solver_type = 'rk2'; %{'Euler' 'default' 'RK2' 'RK4' 'CrankNicolson' 'ImplicitEuler' 'StromerVerlet'}

h  = 60*60*24;                  % Delta t
n  = 2*floor(60*60*24*365.25/h); % rok / Delta t = celkovy pocet kroku simulace
X0 = [x0'; zeros(2*nx,1); v0'];
X  = zeros(4*nx,n+1);  X(:,1) = X0;

function ret__ = F(t,Y)
  nY__  = size(Y,1)/4;
  F__   = zeros(2*nY__,1);
  for i__ = 1:nY__
    for j__ = [1:i__-1, i__+1:nY__]
      r__ = Y([i__,i__+nY__],1)-Y([j__,j__+nY__],1);
      F__([i__,i__+nY__],1) += -G*m(j__)*norm(r__)^(-3)*r__;
    endfor
  endfor
  ret__ = [Y(end/2+1:end,1); F__];
endfunction
% Resime soustavu pro Y(t) = [x;v](t)
% X'(t) = F(t,X(t)) = [v(t); a(t)] kde a(t) = -G*mz*x(t)*norm(x(t))^-3
t = 0;
for i = 1:n+1
  switch lower(solver_type)
    case 'rk2'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h/2,X(:,i)+(h/2)*K1);
      X(:,i+1) = X(:,i) + h * K2; 
    case 'rk4'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h/2,X(:,i)+(h/2)*K1);
      K3 = F(t+h/2,X(:,i)+(h/2)*K2);
      K4 = F(t+h  ,X(:,i)+ h   *K3);
      X(:,i+1) = X(:,i) + h * (K1+2*K2+2*K3+K4)/6; 
    case 'cranknicolson'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h  ,X(:,i)+ h   *K1);
      X(:,i+1) = X(:,i) + h * (K1+K2)/2;
    case 'impliciteuler'
      X(:,i+1) = X(:,i);
      for ii = 1:10
        K = F(t+h,X(:,i+1));
        X(:,i+1) = X(:,i) + h * K;
      end
    case 'irk2'
      t_ = [t + (1/2-sqrt(3)/6)*h; t + (1/2+sqrt(3)/6)*h];
      c_ = [1/4 (1/4-sqrt(3)/6); (1/4+sqrt(3)/6) 1/4];
      K1 = 0;
      K2 = 0;
      for ii = 1:20
        K1 = F(t_(1)    ,X(:,i)+h*(c_(1,1)*K1 + c_(1,2)*K2));
        K2 = F(t_(2)    ,X(:,i)+h*(c_(2,1)*K1 + c_(2,2)*K2));
      end
      X(:,i+1) = X(:,i) + h * (K1/2 + K2/2);
    case 'stormerverlet'
      if i == 1
        FF = F(t,X(:,1));
        X(1:2,2  ) =   X(1:2,1) + h*X(3:4,1) + h^2/2 * FF(3:4);
      else
        FF = F(t,X(:,i));
        X(1:2,i+1) = 2*X(1:2,i) - X(1:2,i-1) + h^2   * FF(3:4);
      end
    otherwise % 'euler'
      K = F(t,X(:,i));
      X(:,i+1) = X(:,i) + h * K; 
  endswitch
  t = t + h;
endfor
figure(1); hold on;
for i = 1:nx
  line(X(i,:), X(i+nx,:), 'linestyle', '-', 'color', co{i});
endfor
axis equal;
endfunction