ms = 1.9885e30;
mz = 5.97237e24;
G  = 6.6743015e-11;
x0 = 1.47098074e11; 
v0 = 3.0287e4;
g  = G*ms;
solver_type = 'irk2'; %{'Euler' 'default' 'RK2' 'RK4' 'CrankNicolson' 'ImplicitEuler' 'StromerVerlet'}

hh = 20;%/24;
h  = 60*60*24*hh;       % jedna hodina
n  = floor(365.25/hh);  % celkovy pocet kroku simulace
X0 = [x0; 0; 0; v0];
X  = zeros(4,n+1);  X(:,1) = X0;
F  = @(t,Y) [Y(3:4,1); -g*Y(1:2,1)*norm(Y(1:2,1))^(-3)];
% Resime soustavu pro Y(t) = [x;v](t)
% X'(t) = F(t,X(t)) = [v(t); a(t)] kde a(t) = -G*mz*x(t)*norm(x(t))^-3
t = 0;
for i = 1:n+1
  switch lower(solver_type)
    case 'rk2'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h/2,X(:,i)+(h/2)*K1);
      X(:,i+1) = X(:,i) + h * K2; 
    case 'rk4'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h/2,X(:,i)+(h/2)*K1);
      K3 = F(t+h/2,X(:,i)+(h/2)*K2);
      K4 = F(t+h  ,X(:,i)+ h   *K3);
      X(:,i+1) = X(:,i) + h * (K1+2*K2+2*K3+K4)/6; 
    case 'cranknicolson'
      K1 = F(t    ,X(:,i)         );
      K2 = F(t+h  ,X(:,i)+ h   *K1);
      X(:,i+1) = X(:,i) + h * (K1+K2)/2;
    case 'impliciteuler'
      X(:,i+1) = X(:,i);
      for ii = 1:10
        K = F(t+h,X(:,i+1));
        X(:,i+1) = X(:,i) + h * K;
      end
    case 'irk2'
      t_ = [t + (1/2-sqrt(3)/6)*h; t + (1/2+sqrt(3)/6)*h];
      c_ = [1/4 (1/4-sqrt(3)/6); (1/4+sqrt(3)/6) 1/4];
      K1 = 0;
      K2 = 0;
      for ii = 1:20
        K1 = F(t_(1)    ,X(:,i)+h*(c_(1,1)*K1 + c_(1,2)*K2));
        K2 = F(t_(2)    ,X(:,i)+h*(c_(2,1)*K1 + c_(2,2)*K2));
      end
      X(:,i+1) = X(:,i) + h * (K1/2 + K2/2);
    case 'stormerverlet'
      if i == 1
        FF = F(t,X(:,1));
        X(1:2,2  ) =   X(1:2,1) + h*X(3:4,1) + h^2/2 * FF(3:4);
      else
        FF = F(t,X(:,i));
        X(1:2,i+1) = 2*X(1:2,i) - X(1:2,i-1) + h^2   * FF(3:4);
      end
    otherwise % 'euler'
      K = F(t,X(:,i));
      X(:,i+1) = X(:,i) + h * K; 
  end
  t = t + h;
end
figure(1); hold on;
plot( 0,0,'xr');
plot(x0,0,'xb');
line(X(1,:),X(2,:));
axis equal;